﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2010/09/28 初版
//
// FPM2 Search Sample Program
// -----------------------------------------------------------------
namespace FPM2_Search_Sample
{
    public partial class Form_FPM2_SEARCH_SAMPLE : Form
    {
        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //ビデオ入力クラス
        private FVIL.Video.CFviVideo m_video = null;
        //特徴点応用マッチングクラス
        private FVIL.FPM2.CFviFPM2Matching m_matching = new FVIL.FPM2.CFviFPM2Matching();
        //FPM2パラメータクラス
        private FVIL.FPM2.CFviFPM2Param m_param = new FVIL.FPM2.CFviFPM2Param();
        private FVIL.FPM2.CFviFPM2ParamAreaScore m_paramA = new FVIL.FPM2.CFviFPM2ParamAreaScore();
        private FVIL.FPM2.CFviFPM2ParamEssential m_paramE = new FVIL.FPM2.CFviFPM2ParamEssential();
        //FPM2テンプレート特徴量抽出クラス
        private FVIL.FPM2.CFviFPM2Template m_template;
        //FPM2マッチング処理モード
        FVIL.FPM2.MatchMode m_matchmode;
        //２次元エッジ検出パラメータクラス(相関エッジ法)
        private FVIL.Edge.CFviEdge2DCorrelationParam m_edgeparam = new FVIL.Edge.CFviEdge2DCorrelationParam();
        //２次元エッジ検出パラメータクラス(Sobel法)
        private FVIL.Edge.CFviEdge2DSobelParam m_sobelparam = new FVIL.Edge.CFviEdge2DSobelParam();
        //パタン構造クラス
        private FVIL.Data.CFviPattern m_pattern = new FVIL.Data.CFviPattern();
        //表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();
        //エッジ表示用オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlayEdge = new FVIL.GDI.CFviOverlay();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();

        //iniファイル設定フォーム
        Form_BROWSEINI formBroseIni = new Form_BROWSEINI();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;

        //時間測定用
        private double msec = 0;

        public Form_FPM2_SEARCH_SAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if (licence_dialog.ShowDialog(this) != DialogResult.OK)
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            // 描画imageのセット
            cFviImageView1.Image = m_image;
            //オーバーレイの追加
            cFviImageView1.Display.Overlays.Add(m_overlay);
            cFviImageView1.Display.Overlays.Add(m_overlayEdge);

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;
            //FPM 特徴点応用 マッチング処理モードの初期化
            m_matchmode = FVIL.FPM2.MatchMode.HighSpeed;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //FPM2コンボボックス初期化
            comboBoxCompressionLevel.Items.Clear();
            comboBoxCompressionLevel.Items.Add("0");
            comboBoxCompressionLevel.Items.Add("1");
            comboBoxCompressionLevel.Items.Add("2");
            comboBoxCompressionLevel.Items.Add("3");
            comboBoxCompressionLevel.Items.Add("4");
            comboBoxCompressionLevel.Items.Add("5");
            comboBoxCompressionLevel.SelectedIndex = 3;

            //FPM2フィルタ選択コンボボックス初期化
            comboBoxSelectFilter.Items.Clear();
            comboBoxSelectFilter.Items.Add("相関エッジ");
            comboBoxSelectFilter.Items.Add("ソーベル");
            comboBoxSelectFilter.SelectedIndex = 0;

            //FPM2エッジ極性選択コンボボックス初期化
            comboBoxPolarity.Items.Clear();
            comboBoxPolarity.Items.Add("極性なし");
            comboBoxPolarity.Items.Add("極性反転");
            comboBoxPolarity.Items.Add("両極性");
            comboBoxPolarity.SelectedIndex = 0;
        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイルの選択
            m_strIniFilePath = "";
            if (formBroseIni.ShowDialog(this) == DialogResult.OK)
            {
                m_strIniFilePath = formBroseIni.strIniFilePath;
            }

            if (m_strIniFilePath.Length > 0)
            {
                try
                {
                    //設定ファイルのロードとデバイスオープン
                    m_video = FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    //画像サイズ調整
                    m_video.ChangeImageSize(m_image);
                    m_flgVideoOpened = true;

                    //ステータス表示の更新
                    //トリガモードの取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;

                    //カメラ露光時間の取得
                    if (m_video.DeviceType != FVIL.Video.DeviceType.RICE001)
                    {
                        NumericUpDownExposure.Value = m_video.ExposureTime;
                    }
                    else
                    {
                        tabControlParam.TabPages.Remove(tabPageBoard);
                    }

                    ButtonGrabOne.Enabled = true;
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (formBroseIni.UseFileFlg == true)
            {
                ButtonGrabOne.Enabled = false;
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            try
            {
                m_overlay.Figures.Clear();
                formBroseIni.UseFileFlg = false;

                //デバイスタイプ
                FVIL.Video.CFviContinuousGrab contgrab;
                switch (m_video.DeviceType)
                {
                    case FVIL.Video.DeviceType.FHC3313:
                        contgrab = ((FVIL.Video.CFviVideoFHC3313)m_video).ContinuousGrab;
                        ((FVIL.Video.CFviContinuousGrabFHC3313)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    default:
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        m_video.GrabImageSync(m_image);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                }
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");

                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// LoadFile ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 入力画像ファイルのロード
        /// </remarks>
        private void buttonLoadImageFile_Click(object sender, EventArgs e)
        {
            formBroseIni.UseFileFlg = true;
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            openFileDialog1.Filter += "|Png ファイル (*.png)|*.png";
            openFileDialog1.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            openFileDialog1.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            openFileDialog1.DefaultExt = "*.bmp";
            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
            {
                return;
            }

            try
            {
                FVIL.File.Function.LoadImageFile(openFileDialog1.FileName, m_image, FVIL.PixelMode.Unpacking);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// サーチ実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// サーチ処理・結果表示
        /// </remarks>
        private void buttonMatching_Click(object sender, EventArgs e)
        {
            //表示初期化
            m_overlay.Figures.Clear();
            listViewResult.Items.Clear();
            //再描画
            cFviImageView1.Refresh();

            //パラメータ設定
            if (fnFPMPraramSet())
            {
                //サーチ実行
                if (fnFPMExec())
                {
                    //結果表示
                    fnResultDisplay();
                }
            }
        }

        /// <summary>
        /// サーチ実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// マッチング処理実行関数
        /// </remarks>
        private bool fnFPMExec()
        {
            bool Result = false;
            try
            {
                FVIL.FPM2.CFviFPM2Feature feature;
                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //FPM2 特徴量(処理対象画像)生成
                    feature = new FVIL.FPM2.CFviFPM2Feature(m_image, m_matchmode, m_edgeparam, checkBoxBorderFlg.Checked);
                }
                else
                {
                    //FPM2 特徴量(処理対象画像)生成
                    feature = new FVIL.FPM2.CFviFPM2Feature(m_image, m_matchmode, m_sobelparam, checkBoxBorderFlg.Checked);
                }
                m_matching.Target = feature;            //特徴量(処理対象画像)

                //時間測定開始
                m_timer.Start();
                //画像処理実行
                m_matching.Execute();
                //時間測定終了
                msec = m_timer.Stop();
                Result = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("サーチエラーが発生しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return (Result);
        }

        /// <summary>
        /// パラメータ設定
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パラメータ設定関数
        /// </remarks>
        private bool fnFPMPraramSet()
        {
            int temp1, temp2;
            bool Result = false;

            //件数
            m_param.Count = (int)numericUpDownCount.Value;
            temp1 = (int)numericUpDownAngleMin.Value;
            temp2 = (int)numericUpDownAngleMax.Value;
            if (temp1 <= temp2)
            {
                //回転角(下限)
                m_param.AngleMin = new FVIL.Data.CFviAngle((double)numericUpDownAngleMin.Value);
                //回転角(上限)
                m_param.AngleMax = new FVIL.Data.CFviAngle((double)numericUpDownAngleMax.Value);
            }
            else
            {
                MessageBox.Show("回転角の上下限が反転しています", "Parameter Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (Result);
            }
            temp1 = (int)numericUpDownScaleMin.Value;
            temp2 = (int)numericUpDownScaleMax.Value;
            if (temp1 <= temp2)
            {
                //スケール(下限)
                m_param.ScaleMin = (int)numericUpDownScaleMin.Value;
                //スケール(上限)
                m_param.ScaleMax = (int)numericUpDownScaleMax.Value;
            }
            else
            {
                MessageBox.Show("スケールの上下限が反転しています", "Parameter Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (Result);
            }
            //圧縮レベル
            m_param.CompressionLevel = comboBoxCompressionLevel.SelectedIndex;
            //誤差範囲
            m_param.ErrorRange = (int)numericUpDownErrorRange.Value;
            //スコア閾値(低圧縮処理用)
            m_param.ScoreThresholdL = (int)numericUpDownScoreThresholdL.Value;
            //スコア閾値(高圧縮処理用)
            m_param.ScoreThresholdH = (int)numericUpDownScoreThresholdH.Value;
            //高精度ポーズ推定の実行指示
            m_param.RefineEnable = checkBoxRefineFlag.Checked;
            switch (comboBoxPolarity.SelectedIndex)
            {
                case 0:
                    m_param.Polarity = FVIL.FPM2.Polarity.Same;         //エッジ極性：極性なし
                    break;
                case 1:
                    m_param.Polarity = FVIL.FPM2.Polarity.Reverse;      //エッジ極性：極性反転
                    break;
                case 2:
                    m_param.Polarity = FVIL.FPM2.Polarity.Either;       //エッジ極性：両極性
                    break;
            }
            //FPM2 領域スコア算出用パラメータ設定
            //誤差範囲(X方向並進値)
            m_paramA.ErrorRangeTx = double.Parse(textBoxAreaErrorRangeTx.Text.ToString());
            //誤差範囲(Y方向並進値)
            m_paramA.ErrorRangeTy = double.Parse(textBoxAreaErrorRangeTy.Text.ToString());
            //誤差範囲(回転角)
            m_paramA.ErrorRangeTq = double.Parse(textBoxAreaErrorRangeTq.Text.ToString());
            //誤差範囲(スケール)
            m_paramA.ErrorRangeTs = double.Parse(textBoxAreaErrorRangeTs.Text.ToString());
            //誤差範囲
            m_paramA.ErrorRange = (int)numericUpDownAreaErrorRange.Value;
            //ノイズデータ重み係数
            m_paramA.NoiseWeight = double.Parse(textBoxAreaNoiseWeight.Text.ToString());
            //スコア閾値
            m_paramA.ScoreThreshold = (int)numericUpDownAreaScoreThreshold.Value;
            //FPM2 不可欠領域スコア算出用パラメータ設定
            //誤差範囲
            m_paramE.ErrorRange = (int)numericUpDownEssenErrorRange.Value;
            //スコア閾値
            m_paramE.ScoreThreshold = (int)numericUpDownEssenScoreThreshold.Value;

            //パタンのエッジ作成
            Result = fnMakePatternEdge();

            m_matching.Param = m_param;             //基本パラメータ
            m_matching.ParamAreaScore = m_paramA;   //領域スコア算出
            m_matching.ParamEssential = m_paramE;   //不可欠領域スコア計算
            m_matching.EnableAreaScore = checkBoxAreaScoreFlag.Checked;     //領域スコア算出の実行指示
            m_matching.EnableEssential = checkBoxEssentialFlg.Checked;      //不可欠領域スコア算出の実行指示

            return (Result);
        }

        /// <summary>
        /// 結果表示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 結果表示関数
        /// </remarks>
        private void fnResultDisplay()
        {
            int i;

            //描画クリア
            m_overlay.Figures.Clear();

            //パタン登録時の基準点
            FVIL.Data.CFviPoint offset = m_template.Offset;
            double width = m_template.Width;
            double height = m_template.Height;

            for (i = 0; i < m_matching.ResultSize; i++)
            {
                FVIL.FPM2.CFviFPM2Data data = m_matching[i];

                //マッチング結果の取得
                FVIL.Data.CFviPoint pos = data.Position;    //回答位置
                FVIL.Data.CFviAngle angle = data.Angle;     //回転角
                double scale = data.Scale;                  //スケール(％)
                int score = data.Score;                     //スコア(％)

                //矩形
                FVIL.Data.CFviRectangle rect = new FVIL.Data.CFviRectangle();
                rect.St = new FVIL.Data.CFviPoint(pos.X - offset.X, pos.Y - offset.Y);
                rect.Ed = new FVIL.Data.CFviPoint(rect.St.X + width, rect.St.Y + height);
                rect.Center = offset;
                rect.Angle = angle;

                //描画用
                //矩形
                FVIL.GDI.CFviGdiRectangle ol_rect = new FVIL.GDI.CFviGdiRectangle(rect);
                ol_rect.Pen.Color = Color.FromArgb(0x00, 0x00, 0xFF);
                ol_rect.FillEnable = true;
                ol_rect.FillAlpha = 0x3F;
                ol_rect.FillColor = System.Drawing.Color.FromArgb(0xFF, 0x00, 0x00);

                //回答位置
                FVIL.GDI.CFviGdiPoint ol_pos = new FVIL.GDI.CFviGdiPoint(pos);
                ol_pos.Size = new Size(5, 5);
                ol_pos.Pen.Color = Color.FromArgb(0xFF, 0x00, 0x00);
                ol_pos.Angle = rect.Angle;

                //スコア
                FVIL.GDI.CFviGdiString ol_score = new FVIL.GDI.CFviGdiString();
                ol_score.Text = scale.ToString();
                ol_score.Position = pos;
                ol_score.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Bottom;
                ol_score.BkMode = FVIL.GDI.BkMode.Transparent;
                ol_score.Color = Color.FromArgb(0x00, 0xFF, 0x00);
                ol_score.Angle = rect.Angle;

                //スケール
                FVIL.GDI.CFviGdiString ol_scale = new FVIL.GDI.CFviGdiString();
                ol_scale.Text = scale.ToString("0");
                ol_scale.Position = pos;
                ol_scale.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Top;
                ol_scale.BkMode = FVIL.GDI.BkMode.Transparent;
                ol_scale.Color = Color.FromArgb(0x7F, 0xFF, 0x00);
                ol_scale.Angle = rect.Angle;

                //傾き
                FVIL.GDI.CFviGdiString ol_angle = new FVIL.GDI.CFviGdiString();
                ol_angle.Text = rect.Angle.Degree.ToString("0");
                ol_angle.Position = new FVIL.Data.CFviPoint(pos.X, pos.Y + ol_scale.Font.Height);
                ol_angle.Align = FVIL.GDI.TextAlign.Right | FVIL.GDI.TextAlign.Top;
                ol_angle.BkMode = FVIL.GDI.BkMode.Transparent;
                ol_angle.Color = Color.FromArgb(0x00, 0xFF, 0xFF);
                ol_angle.Angle = rect.Angle;
                ol_angle.Axis = new FVIL.Data.CFviPoint(0, -ol_scale.Font.Height);

                //追加
                m_overlay.Figures.Add(ol_rect);
                m_overlay.Figures.Add(ol_pos);
                m_overlay.Figures.Add(ol_score);
                m_overlay.Figures.Add(ol_scale);
                m_overlay.Figures.Add(ol_angle);

                //リスト表示
                if (i == 0)
                {
                    string[] listitem = { msec.ToString("0.000"), i.ToString(), data.Position.X.ToString("0.00"), data.Position.Y.ToString("0.00"), data.Angle.Degree.ToString("0.00"), data.Scale.ToString("0.00"), data.Score.ToString() };
                    listViewResult.Items.Add(new ListViewItem(listitem));
                }
                else
                {
                    string[] listitem = { "", i.ToString(), data.Position.X.ToString("0.00"), data.Position.Y.ToString("0.00"), data.Angle.Degree.ToString("0.00"), data.Scale.ToString("0.00"), data.Score.ToString() };
                    listViewResult.Items.Add(new ListViewItem(listitem));
                }
            }
            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// パタン読込ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン画像のロード
        /// </remarks>
        private void buttonLoadPattern_Click(object sender, EventArgs e)
        {
            //入力画像ファイルのロード
            openFileDialog1.FileName = null;
            openFileDialog1.Filter = "Pnt ファイル (*.pat)|*.pat";
            openFileDialog1.DefaultExt = "*.pat";
            if ((openFileDialog1.FileNames.Length <= 0) && (openFileDialog1.ShowDialog(this) == DialogResult.Cancel))
            {
                return;
            }

            try
            {
                m_pattern.Load(openFileDialog1.FileName);

                //パタン表示ボタン有効化
                buttonDisplayPattern.Enabled = true;
                //パタン保存ボタン有効化
                buttonSavePattern.Enabled = true;
                //マスク設定ボタン有効化
                buttonDisplayMask.Enabled = true;
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// パタン登録ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン登録用フォーム表示
        /// </remarks>
        private void buttonSetPattern_Click(object sender, EventArgs e)
        {
            //画像表示フォーム
            Form_ProcDISPLAY formProcDisplay = new Form_ProcDISPLAY();
            formProcDisplay.Text = "Set Pattern";

            //パタンデータ設定
            formProcDisplay.Set_image(m_image);
            formProcDisplay.Set_pattern(m_pattern);
            formProcDisplay.Set_OverlayPattern();

            //画像表示画面
            formProcDisplay.ShowDialog(this);

            //パタン表示ボタン有効化
            buttonDisplayPattern.Enabled = true;
            //パタン保存ボタン有効化
            buttonSavePattern.Enabled = true;
            //マスク設定ボタン有効化
            buttonDisplayMask.Enabled = true;
        }

        /// <summary>
        /// パタンのエッジ作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン画像の特徴量を抽出する関数
        /// </remarks>
        private bool fnMakePatternEdge()
        {
            try
            {
                FVIL.FPM2.MatchMode matchmode;
                if (checkBoxMatchMode.Checked == true)
                {
                    matchmode = FVIL.FPM2.MatchMode.HighSpeed;
                }
                else
                {
                    matchmode = FVIL.FPM2.MatchMode.Normal;
                }

                if (comboBoxSelectFilter.SelectedIndex == 0)
                {
                    //FPM2 (相関エッジ)特徴量抽出用パラメータ設定
                    int temp1 = (int)numericUpDownRegionHeight.Value;
                    int temp2 = (int)numericUpDownRegionWidth.Value;
                    if ((temp1 <= temp2) && ((temp1 * temp2) <= 255))
                    {
                        m_edgeparam.RegionWidth = (int)numericUpDownRegionWidth.Value;
                        m_edgeparam.RegionHeight = (int)numericUpDownRegionHeight.Value;
                    }
                    else
                    {
                        MessageBox.Show("局所領域サイズの条件を満たしていません", "Parameter Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return(false);
                    }
                    m_edgeparam.SigmoidK = double.Parse(textBoxSigmoidK.Text.ToString());
                    m_edgeparam.VarThreshold = double.Parse(textBoxVarThreshold.Text.ToString());
                    m_edgeparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                    m_edgeparam.NmsLength = (int)numericUpDownNmsLength.Value;
                    //FPM2 特徴量(テンプレート)生成
                    m_template = new FVIL.FPM2.CFviFPM2Template(m_pattern, matchmode, m_edgeparam);
                }
                else
                {
                    //FPM2 (ソーベル)特徴量抽出用パラメータ設定
                    m_sobelparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                    m_sobelparam.NmsLength = (int)numericUpDownNmsLength.Value;
                    //FPM2 特徴量(テンプレート)生成
                    m_template = new FVIL.FPM2.CFviFPM2Template(m_pattern, matchmode, m_sobelparam);
                }
                m_matching.Template = m_template;       //特徴量(テンプレート)
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("特徴量の生成に失敗しました。ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return (false);
            }
            return (true);
        }

        /// <summary>
        /// パタン保存ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン画像の保存
        /// </remarks>
        private void buttonSavePattern_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();
            ofdlg.FileName = "Pattern.pat";
            ofdlg.Filter = "Pat ファイル (*.pat)|*.pat";
            ofdlg.DefaultExt = "*.pat";
            ofdlg.OverwritePrompt = true;
            ofdlg.ShowDialog();
            if (ofdlg.FileNames.Length <= 0)
            {
                return;
            }

            try
            {
                //パタン画像の保存
                m_pattern.Save(ofdlg.FileName);
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// FPM2エッジ検出手法選択コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 相関エッジ・ソーベルの選択。
        /// </remarks>
        private void comboBoxSelectFilter_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxSelectFilter.SelectedIndex == 0)
            {
                //相関エッジ用パラメータ表示
                numericUpDownRegionHeight.Visible = true;
                numericUpDownRegionWidth.Visible = true;
                textBoxSigmoidK.Visible = true;
                textBoxVarThreshold.Visible = true;
                labelRegionWidth.Visible = true;
                labelRegionHeight.Visible = true;
                labelSigmoidK.Visible = true;
                labelVarThreshold.Visible = true;
                labelCondition1.Visible = true;
                labelCondition2.Visible = true;
                labelCondition3.Visible = true;
                //相関エッジ用パラメータ初期値設定
                numericUpDownEdgeThreshold.Maximum = 255;
                numericUpDownEdgeThreshold.Value = m_edgeparam.EdgeThreshold;
                numericUpDownNmsLength.Value = m_edgeparam.NmsLength;
            }
            else
            {
                //相関エッジ用パラメータ非表示
                numericUpDownRegionHeight.Visible = false;
                numericUpDownRegionWidth.Visible = false;
                textBoxSigmoidK.Visible = false;
                textBoxVarThreshold.Visible = false;
                labelRegionWidth.Visible = false;
                labelRegionHeight.Visible = false;
                labelSigmoidK.Visible = false;
                labelVarThreshold.Visible = false;
                labelCondition1.Visible = false;
                labelCondition2.Visible = false;
                labelCondition3.Visible = false;
                //ソーベルエッジ用パラメータ初期値設定
                numericUpDownEdgeThreshold.Maximum = 1023;
                numericUpDownEdgeThreshold.Value = m_sobelparam.EdgeThreshold;
                numericUpDownNmsLength.Value = m_sobelparam.NmsLength;
            }
        }

        /// <summary>
        /// 画像エッジ表示チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像のエッジの表示を切替えます。
        /// </remarks>
        private void checkBoxEdgeDisplay_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (checkBoxEdgeDisplay.Checked == true)
                {
                    FVIL.FPM2.CFviFPM2Feature feature;
                    List<FVIL.Edge.CFviEdgeData> edges_image;

                    m_overlayEdge.Figures.Clear();

                    if (comboBoxSelectFilter.SelectedIndex == 0)
                    {
                        int temp1, temp2;

                        //FPM2 (相関エッジ)特徴量抽出用パラメータ設定
                        temp1 = (int)numericUpDownRegionHeight.Value;
                        temp2 = (int)numericUpDownRegionWidth.Value;
                        if ((temp1 <= temp2) && ((temp1 * temp2) <= 255))
                        {
                            m_edgeparam.RegionWidth = (int)numericUpDownRegionWidth.Value;
                            m_edgeparam.RegionHeight = (int)numericUpDownRegionHeight.Value;
                        }
                        else
                        {
                            MessageBox.Show("局所領域サイズの条件を満たしていません", "Parameter Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        m_edgeparam.SigmoidK = double.Parse(textBoxSigmoidK.Text.ToString());
                        m_edgeparam.VarThreshold = double.Parse(textBoxVarThreshold.Text.ToString());
                        m_edgeparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                        m_edgeparam.NmsLength = (int)numericUpDownNmsLength.Value;

                        //FPM2 特徴量(処理対象画像)生成
                        feature = new FVIL.FPM2.CFviFPM2Feature(m_image, m_matchmode, m_edgeparam, checkBoxBorderFlg.Checked);
                    }
                    else
                    {
                        //FPM2 (ソーベル)特徴量抽出用パラメータ設定
                        m_sobelparam.EdgeThreshold = (int)numericUpDownEdgeThreshold.Value;
                        m_sobelparam.NmsLength = (int)numericUpDownNmsLength.Value;

                        //FPM2 特徴量(処理対象画像)生成
                        feature = new FVIL.FPM2.CFviFPM2Feature(m_image, m_matchmode, m_sobelparam, checkBoxBorderFlg.Checked);
                    }

                    //処理対象画像のエッジ点
                    edges_image = (List<FVIL.Edge.CFviEdgeData>)feature.GetFeatureEdges();

                    //エッジの表示
                    for (int i = 0; i < edges_image.Count; i++)
                    {
                        FVIL.Edge.CFviEdgeData edge = edges_image[i];
                        // エッジ位置に ＞ マークを表示 (0度が左向き)
                        FVIL.GDI.CFviGdiPoint position = new FVIL.GDI.CFviGdiPoint(edge.Position);
                        position.Size = new System.Drawing.Size(3, 1);
                        position.Style = FVIL.GDI.FigureStyle.Arrow;
                        position.Angle = edges_image[i].Angle;
                        position.Pen.Color = System.Drawing.Color.FromArgb(0, 0xFF, 0);
                        m_overlayEdge.Figures.Add(position);
                    }
                }
                else
                {
                    m_overlayEdge.Figures.Clear();
                }
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// マッチング高速モード チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// FPM 特徴点応用 マッチング処理モードの設定。
        /// </remarks>
        private void checkBoxMatchMode_CheckedChanged(object sender, EventArgs e)
        {
            //FPM 特徴点応用 マッチング処理モード設定
            if (checkBoxMatchMode.Checked == true)
            {
                m_matchmode = FVIL.FPM2.MatchMode.HighSpeed;
            }
            else
            {
                m_matchmode = FVIL.FPM2.MatchMode.Normal;
            }
        }

        /// <summary>
        /// パタン表示ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// パタン表示用フォーム表示。
        /// </remarks>
        private void buttonDisplayPattern_Click(object sender, EventArgs e)
        {
            //パタン表示フォーム
            Form_PTNDISPLAY formPtnDisplay = new Form_PTNDISPLAY();

            //パタンのエッジ作成
            fnMakePatternEdge();
            //パタンデータ設定
            formPtnDisplay.Set_pattern(m_pattern);
            formPtnDisplay.Set_template(m_template);

            //パタン表示フォームの表示をパタン表示用に変更
            formPtnDisplay.Text = "Pattern DISPLAY";
            formPtnDisplay.Mode_Pattern();

            //パタン表示画面
            formPtnDisplay.ShowDialog(this);
        }

        /// <summary>
        /// マスク設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// マスク設定用フォーム表示。
        /// </remarks>
        private void buttonDisplayMask_Click(object sender, EventArgs e)
        {
            //パタン表示フォーム
            Form_PTNDISPLAY formPtnDisplay = new Form_PTNDISPLAY();

            //パタンのエッジ作成
            fnMakePatternEdge();
            //パタンデータ設定
            formPtnDisplay.Set_pattern(m_pattern);
            formPtnDisplay.Set_template(m_template);

            //パタン表示フォームの表示をマスク設定用に変更
            formPtnDisplay.Text = "Pattern Mask DISPLAY";
            formPtnDisplay.Mode_Mask();

            //パタン表示画面
            formPtnDisplay.ShowDialog(this);
        }

        /// <summary>
        /// 処理範囲設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 処理範囲設定用フォーム表示。
        /// </remarks>
        private void buttonRegion_Click(object sender, EventArgs e)
        {
            //画像表示フォーム
            Form_ProcDISPLAY formProcDisplay = new Form_ProcDISPLAY();
            formProcDisplay.Text = "Set Area";

            //画像データ設定
            formProcDisplay.Set_image(m_image);
            formProcDisplay.Set_OverlayProc();

            //画像表示画面
            formProcDisplay.ShowDialog(this);

            //再描画
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// x１ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FPM_1CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }
        
        /// <summary>
        /// X方向並進値テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// X方向並進値の誤差範囲のエラー検出。
        /// </remarks>
        private void textBoxAreaErrorRangeTx_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_paramA.ErrorRangeTx = double.Parse(textBoxAreaErrorRangeTx.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxAreaErrorRangeTx.Text = "0.5";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Y方向並進値テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// Y方向並進値の誤差範囲のエラー検出。
        /// </remarks>
        private void textBoxAreaErrorRangeTy_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_paramA.ErrorRangeTy = double.Parse(textBoxAreaErrorRangeTy.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxAreaErrorRangeTy.Text = "0.5";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 回転角テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 回転角の誤差範囲のエラー検出。
        /// </remarks>
        private void textBoxAreaErrorRangeTq_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_paramA.ErrorRangeTq = double.Parse(textBoxAreaErrorRangeTq.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxAreaErrorRangeTq.Text = "0.5";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// スケールテキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// スケールの誤差範囲のエラー検出。
        /// </remarks>
        private void textBoxAreaErrorRangeTs_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_paramA.ErrorRangeTs = double.Parse(textBoxAreaErrorRangeTs.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxAreaErrorRangeTs.Text = "0.5";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// ノイズデータ重み係数テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 背景領域ノイズデータ重み係数のエラー検出。
        /// </remarks>
        private void textBoxAreaNoiseWeight_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_paramA.NoiseWeight = double.Parse(textBoxAreaNoiseWeight.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxAreaNoiseWeight.Text = "0.2";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// シグモイド関数Kテキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// シグモイド関数のパラメータkのエラー検出。
        /// </remarks>
        private void textBoxSigmoidK_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_edgeparam.SigmoidK = double.Parse(textBoxSigmoidK.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxSigmoidK.Text = "1.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// 分散閾値テキストボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 分散閾値のエラー検出。
        /// </remarks>
        private void textBoxVarThreshold_TextChanged(object sender, EventArgs e)
        {
            try
            {
                m_edgeparam.VarThreshold = double.Parse(textBoxVarThreshold.Text.ToString());
            }
            catch (System.Exception ex)
            {
                //エラー処理
                //数字以外の入力は初期値にする。
                textBoxVarThreshold.Text = "25.0";
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

    }
}